from github import Github

from colorama import Fore, Style

from .utils import CredsManager, salir, info, error


class GithubClient:

    def __init__(self, cfg):
        self.creds = CredsManager()
        self.TOKEN = self.creds.get_token("github")
        self.SRV = cfg.server
        self.USR = cfg.username
        self.REPO = cfg.repo
        self.link = cfg.link

        self.g = Github(self.TOKEN)
        try:
            self.r = self.g.get_repo(f"{self.USR}/{self.REPO}")
        except Exception as e:
            print(e.args)
            salir("Github client can't connect")

    # ─── COMMANDS ───────────────────────────────────────────────────────────────────

    def list_issues(self):
        issues_list = self.open_issues
        for i in issues_list:
            print(f"  {Style.BRIGHT}{i.number} {Style.DIM}{i.title.title()}")
        if len(issues_list) == 0:
            print(f"    {Style.DIM}There no open issues assigned at current Milestone ")

    def close_issue(self, id):
        issue = self.r.get_issue(number=int(id))
        issue.edit(state='closed')

    def show_issue(self, id):
        issue = self.r.get_issue(number=int(id))
        print(issue.url)
        print(issue.title)
        print(issue.body)

    def new_issue(self, labels):
        title = input("Input title: ")
        body = input("Input contents: ")
        self.r.create_issue(title=title, body=body, labels=labels)

    # ─── PROPERTIES ─────────────────────────────────────────────────────────────────

    @property
    def open_issues(self):
        lista = []
        for i in self.r.get_issues():
            if i.milestone is not None:
                if i.milestone.state == "open":
                    lista.append(i)
        return lista

# ────────────────────────────────────────────────────────────────────────────────



import sys

from .utils import SettingsFile, get_version, print_header, salir, info, error
from .github import GithubClient


def get_client():
    # check settings file and create if not exist
    config = SettingsFile()
    if not config.exist:
        config.create()
    # devuelve cliente del server que toca
    if config.server == "github.com":
        return GithubClient(config)
    else:
        salir("Server not implemented")


def run_handler(args):
    print_header()

    if args['--version']:
        info(f"Current version: {get_version()}")
        sys.exit()

    client = get_client()

    if args['--list']:
        info("Listing all open issues from current Milestone...\n")
        client.list_issues()

    elif args['--new']:
        info("Creating new generic isse...")
        client.new_issue([])

    elif args['--bug']:
        info("Creating new bug isse...")
        client.new_issue(['bug'])

    elif args['--show'] and args['<id>'] is not None:
        client.show_issue(args['<id>'])

    elif args['--close'] and args['<id>'] is not None:
        info("Closing issue...")
        client.close_issue(args['<id>'])

    print("")



'''
Git Manager Tool

Usage:
  gm -l | --list
  gm -n | --new
  gm -b | --bug
  gm -s | --show <id>
  gm -c | --close <id>
  gm -h | --help
  gm -v | --version

Options:
  -l --list     Show issues list from current Milestone
  -n --new     Create new issue without label
  -b --bug     Create new issue as bug
  -s --show     Show an issue specifying his ID
  -c --close     Close an issue specifying his ID
  -h --help     Show this screen.
  -v --version     Show version.
'''

import sys
from docopt import docopt
from colorama import init
from .handler import run_handler


# ─── MAIN ───────────────────────────────────────────────────────────────────────

def main():
    init(autoreset=True)  # init colorama
    arguments = docopt(__doc__)

    try:
        run_handler(arguments)
    except KeyboardInterrupt:
        sys.exit("Exiting...")
    except Exception as e:
        print(e)



import yaml
import os
import sys

from colorama import Fore, Style

VERSION = 0.6


# ─── SETTINGS CLASS ─────────────────────────────────────────────────────────────

class SettingsFile:

    NAME = "settings.gm"

    def create(self):
        print("Settings file not exist, creating...")
        link = input("Input link of git server: ")
        with open(self.NAME, 'w') as f:
            yaml.dump({"link": link}, f, default_flow_style=False)

    @property
    def link(self):
        try:
            with open(self.NAME, "r") as f:
                data = yaml.load(f, Loader=yaml.FullLoader)
        except Exception as e:
            print(f"error: {e}")
            sys.exit()
        else:
            return data['link']

    @property
    def exist(self):
        return os.path.isfile(self.NAME)

    @property
    def server(self):
        return self.link.split("/")[2]

    @property
    def username(self):
        return self.link.split("/")[3]

    @property
    def repo(self):
        return self.link.split("/")[4]


# ────────────────────────────────────────────────────────────────────────────────


# ─── CREDENTIALS MANAGER ────────────────────────────────────────────────────────

# Clase encargada de gestionar el fichero de credenciales
class CredsManager:

    CREDS_FOLDER = os.environ["HOME"] + "/.gitmanager"
    CREDS_FILE = "creds.yaml"

    def __init__(self):
        self.CREDS_PATH = f"{self.CREDS_FOLDER}/{self.CREDS_FILE}"
        if not self.exist:
            self._generate_basic_file()

    def _generate_basic_file(self):
        try:
            print(" [i] Creating credentials file...")
            os.system(f"mkdir -p {self.CREDS_FOLDER}")
            os.system(f"touch {self.CREDS_PATH}")
        except Exception as e:
            print(e)
            sys.exit(" [!] Can't create the credentials file")
        else:
            print("\n [i] Adding servers...")
            tokens = {
                "github": self.input_token("github"),
                "gitlab": self.input_token("gitlab")
            }
            with open(self.CREDS_PATH, 'w') as f:
                yaml.dump(tokens, f, default_flow_style=False)

    @property
    def exist(self):
        return os.path.isfile(self.CREDS_PATH)

    def get_token(self, server):
        with open(self.CREDS_PATH, "r") as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
            if data[server] is None:
                print(" [!] Please add token on config file:")
                print(f"\t{self.CREDS_PATH}")
                sys.exit()
            return data[server]

    def input_token(self, server):
        token = input(f"   Input token for {server.title()}: ")
        if len(token) == 0:
            return None
        return token

# ────────────────────────────────────────────────────────────────────────────────


# return current version
def get_version():
    # try:
    #     with open('gitmanager/version.yaml', "r") as f:
    #         data = yaml.load(f, Loader=yaml.FullLoader)
    # except Exception as e:
    #     print(e)
    # else:
    #     return data['version']
    return VERSION


def print_header():
    print(f"{Fore.GREEN}{Style.DIM}# Git Manager Tool (v{get_version()})\n\n")


def salir(mensaje=None):
    if mensaje is not None:
        print(f"{Fore.RED}[X]{Fore.RESET} {mensaje}\n")
    sys.exit("Exiting...")


def info(mensaje):
    print(f"{Fore.BLUE}[i]{Fore.RESET} {mensaje}")


def error(mensaje):
    print(f" {Fore.RED}[i]{Fore.RESET} {mensaje}")



import setuptools
from gitmanager.utils import get_version

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="gitmantool",
    version=str(get_version()),
    author='katmai',
    author_email='katmai.mobil@gmail.com',
    description='Console tool for manage projects on github/gitlab',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/katmai1/gitmanager",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'gm=gitmanager:main',
        ],
    },
    install_requires=[
        'docopt', 'pygithub', 'gitlab', 'pyyaml', 'colorama'
    ],
)
